// HtmlFrame.java

package uk.org.redfelineninja.awt;

import java.awt.BorderLayout;
import java.io.*;
import java.net.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;

/**
 * HtmlFrame is a simple class to display any supplied HTML document as a
 * separate window. It was written to allow developers to very rapidly put
 * on-line help into any Swing applet or application.
 * <p>
 * Only the constructors should be used by these applets or applications.
 */
public class HtmlFrame extends JFrame implements HyperlinkListener {
    public HtmlFrame(File f) throws IOException {
        this(f, f.toString());
    }

    public HtmlFrame(URL u) throws IOException {
        this(u, u.toString());
    }

    public HtmlFrame(File f, String title) throws IOException {
        super(title);

        try {
            URL u = new URL("file:" + f.getAbsolutePath());
            contructorHelper(u);
        } catch (MalformedURLException e) {
            System.err.println("Can not open file '" + f.getAbsolutePath() + "'");
            this.setVisible(false);
        }

    }

    public HtmlFrame(URL u, String title) throws IOException {
        super(title);
        contructorHelper(u);
    }

    JEditorPane html;

    private void contructorHelper(URL url) throws IOException {
        html = new JEditorPane(url);

        html.setEditable(false);
        html.addHyperlinkListener(this);

        JScrollPane scroller = new JScrollPane();

        JViewport vp = scroller.getViewport();
        vp.add(html);
        vp.setScrollMode(JViewport.BLIT_SCROLL_MODE);

        getContentPane().setLayout(new BorderLayout());
        getContentPane().add(scroller, BorderLayout.CENTER);

        setSize(512, 384);
        setVisible(true);
    }

    public void hyperlinkUpdate(HyperlinkEvent e) {
        if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
            // load a new doc.

            // back up old doc
            Document doc = html.getDocument();

            // try to load the new url
            try {
                html.setPage(e.getURL());
            } catch (IOException ioe) {
                html.setDocument(doc);
                getToolkit().beep();
            }

            // redraw
            invalidate();
            repaint();
        }
    }
} // HtmlFrame.class
