// JTextAreaWriter.java

package uk.org.redfelineninja.awt;

import javax.swing.*;
import java.io.*;

/**
 * This Writer can be connected directly to a JTextArea to provide writer
 * services directly into a GUI.
 *
 * @see javax.swing.JTextArea
 */
public class JTextAreaWriter extends Writer {
    private JTextArea textarea;
    private JScrollBar vBar;

    /**
     * Creates a new JTextAreaWriter.
     *
     * The editable flag of the JTextArea will be set to false as it
     * should not normally be edited by the user.
     */
    public JTextAreaWriter(JTextArea ta, JScrollPane scroll) {
        textarea = ta;
        textarea.setEditable(false);

        vBar = scroll.getVerticalScrollBar();
    }

    /**
     * Set a new JTextArea to write to.
     */
    public void setTextArea(JTextArea ta) {
        textarea = ta;
        textarea.setEditable(false);
    }

    /**
     * Get the JTextArea that the writer uses.
     *
     * @return the JTextArea used in this Writer
     */
    public JTextArea getTextArea() {
        return textarea;
    }

    public void write(char cbuf[], int off, int len) throws IOException {
        textarea.append(new String(cbuf, off, len));
    }

    public void flush() {
        textarea.repaint();

        vBar.setValue(vBar.getMaximum());
    }

    public void close() {
        // forget the object
        textarea = null;
    }

} // class
