/*
 * This class was originally supplied in the demos for Java2. It has been
 * seperated from its original filename, MetalworksDocumentFrame.java.
 *
 * It is therefore copyrighted to Sun Microsystems.
 *
 * Modified by Daniel Thompson
 */

package uk.org.redfelineninja.awt;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import javax.swing.*;

/**
 * This Lays out components as pairs, it is much more pleasant than a
 * GridBagLayout.
 *
 * Typical uses might be in options dialog.
 * <p>
 * Add methods should be given two parameters <tt>add( Component, String )</tt>
 * where String takes either <tt>label</tt> or <tt>field</tt>.
 *
 * @author Steve Wilson (code), Sun Microsystems.
 * @author Daniel Thompson (doc comments)
 */
public class LabeledPairLayout implements LayoutManager {

    Vector labels = new Vector();
    Vector fields = new Vector();

    int yGap = 2;
    int xGap = 2;

    public void addLayoutComponent(String s, Component c) {
        if (s.equals("label")) {
            labels.addElement(c);
        } else {
            fields.addElement(c);
        }
    }

    public void layoutContainer(Container c) {
        Insets insets = c.getInsets();

        int labelWidth = 0;
        Enumeration labelIter = labels.elements();
        while (labelIter.hasMoreElements()) {
            JComponent comp = (JComponent) labelIter.nextElement();
            labelWidth = Math.max(labelWidth, comp.getPreferredSize().width);
        }

        int yPos = insets.top;

        Enumeration fieldIter = fields.elements();
        labelIter = labels.elements();
        while (labelIter.hasMoreElements() && fieldIter.hasMoreElements()) {
            JComponent label = (JComponent) labelIter.nextElement();
            JComponent field = (JComponent) fieldIter.nextElement();
            int height = Math.max(label.getPreferredSize().height, field.getPreferredSize().height);
            label.setBounds(insets.left, yPos, labelWidth, height);
            field.setBounds(
                insets.left + labelWidth + xGap,
                yPos,
                c.getSize().width - (labelWidth + xGap + insets.left + insets.right),
                height);
            yPos += (height + yGap);
        }

    }

    public Dimension minimumLayoutSize(Container c) {
        Insets insets = c.getInsets();

        int labelWidth = 0;
        Enumeration labelIter = labels.elements();
        while (labelIter.hasMoreElements()) {
            JComponent comp = (JComponent) labelIter.nextElement();
            labelWidth = Math.max(labelWidth, comp.getPreferredSize().width);
        }

        int yPos = insets.top;

        labelIter = labels.elements();
        Enumeration fieldIter = fields.elements();
        while (labelIter.hasMoreElements() && fieldIter.hasMoreElements()) {
            JComponent label = (JComponent) labelIter.nextElement();
            JComponent field = (JComponent) fieldIter.nextElement();
            int height = Math.max(label.getPreferredSize().height, field.getPreferredSize().height);
            yPos += (height + yGap);
        }
        return new Dimension(labelWidth * 3, yPos);
    }

    public Dimension preferredLayoutSize(Container c) {
        Dimension d = minimumLayoutSize(c);
        d.width *= 2;
        return d;
    }

    public void removeLayoutComponent(Component c) {}

}
