// HashColorModel.java

package uk.org.redfelineninja.image;

import java.awt.image.*;
import java.util.Hashtable;

public class HashColorModel extends ColorModel {
    // DATA MEMBERS

    /**
     * Translates certain colors from one value to another.
     *
     * Contains only java.lang.Integer so casting is safe without checks
     */
    Hashtable colors = new Hashtable();

    // CONTRUCTOR

    public HashColorModel() {
        super(32);
    }

    // MANIPULATION

    public void addTranslation(int from, int to) {
        colors.put(new Integer(from), new Integer(to));
    }

    public void removeTranslation(int from) {
        colors.remove(new Integer(from));
    }

    // TRANSLATIONS

    public int getRed(int pixel) {
        return (getRGB(pixel) >> 16) & 0xff;
    }

    public int getGreen(int pixel) {
        return (getRGB(pixel) >> 8) & 0xff;
    }

    public int getBlue(int pixel) {
        return getRGB(pixel) & 0xff;
    }

    public int getAlpha(int pixel) {
        return (getRGB(pixel) >> 24) & 0xff;
    }

    public int getRGB(int pixel) {
        // this is safe because colors only contains java.lang.Integer
        Integer theColor = (Integer) colors.get(new Integer(pixel));

        if (theColor == null)
            return pixel;
        else
            return theColor.intValue();
    }
} // class
