// MulticastWriter.java

package uk.org.redfelineninja.net;

import java.io.*;

/**
 * Tool for broadcasting actions performed on groups of writers.
 */
public class MulticastWriter extends Writer {
    GenericMulticaster multicaster;

    /**
     * Create a new MutlicastWriter with no listeners attached.
     */
    public MulticastWriter() {
        multicaster = new GenericMulticaster();
    }

    /**
     * Add a new listener to this multicaster. The listener must be a writer.
     */
    public void addListener(Writer w) {
        multicaster.addListener(w);
    }

    /**
     * Remove listener from this multicaster. The listener must be a writer.
     */
    public void removeListener(Writer w) {
        multicaster.removeListener(w);
    }

    /**
     * Write a portion of an array of characters to all listeners.
     */
    public void write(char[] cbuf, int off, int len) throws IOException {
        Object[] args = new Object[3];
        args[0] = cbuf;
        args[1] = new Integer(off);
        args[2] = new Integer(len);
        multicaster.invokeMethod("write", args);
    }

    /**
     * Flush the stream of all listeners.
     */
    public void flush() throws IOException {
        multicaster.invokeMethod("flush", null);
    }

    /**
     * Close the stream of all listeners, flushing them first.
     */
    public void close() throws IOException {
        multicaster.invokeMethod("close", null);
    }

    /* *
     * Test code to test spelling etc
     * /
    public static void main( String args[] ) {
       // Test
       Writer w = new OutputStreamWriter( System.out );
    
       MulticastWriter mw = new MulticastWriter();
    
       try {
          mw.addListener( w );
    	 mw.write( "Once\n" );
    
    	 mw.addListener( w );
    	 mw.write( "Twice\n" );
    
    	 mw.addListener( w );
    	 mw.write( "Thrice\n" );
    
    	 mw.removeListener( w );
    	 mw.write( "Twice\n" );
    
          mw.flush();
    	 mw.close();
       }
       catch( Exception e ) {
          e.printStackTrace();
       }
    }
    /**/
} // MulticastWriter.java
