// RiskNetworkDialogListener.java

package uk.org.redfelineninja.risk.client;

import uk.org.redfelineninja.risk.data.*;
import uk.org.redfelineninja.risk.net.*;
import uk.org.redfelineninja.risk.server.RiskServer;

import java.awt.event.*;
import java.net.*;
import java.io.*;

import javax.swing.JOptionPane;

/**
 * This class can listen to the Network dialog box, interogate it and make the
 * appropriate network connections.
 */
public class RiskNetworkDialogListener implements ActionListener {RiskServerInterface server;

    public RiskNetworkDialogListener(RiskServerInterface s) {
        server = s;
    }

    public void actionPerformed(ActionEvent e) {
        boolean startServerAndRetry = false;

        // lets be carefull
        if (!(e.getSource() instanceof RiskNetworkDialog))
            throw new RuntimeException("RiskNeworkDialogListener abused");

        RiskNetworkDialog dialog = (RiskNetworkDialog) e.getSource();

        // create an active (true) player
        Player player = new Player(dialog.getPlayerName(), dialog.getPlayerColor(), true);

        // make a connection to the server if required
        if (!server.isConnected()) {
            try {
                server.openConnection(dialog.getHostname(), dialog.getPort(), player);
            } catch (UnknownHostException uhe) {
                String title = Words.unknownHost + dialog.getHostname();
                JOptionPane.showMessageDialog(dialog, Words.unknownHostFurther, title, JOptionPane.WARNING_MESSAGE);
            } catch (ConnectException ce) {
                startServerAndRetry = true;
            } catch (IOException ioe) {
                // I do not really expect this one!
                ioe.printStackTrace();
            }

            if (startServerAndRetry) {
                try {
                    // start up a new risk server
                    String[] serverArgs = new String[1];
                    serverArgs[0] = "--noblock";
                    RiskServer.main(serverArgs);

                    // ensure the server has time to start up
                    Thread.sleep(100);

                    // try again to connect
                    server.openConnection(dialog.getHostname(), dialog.getPort(), player);
                } catch (ConnectException ce) {
                    String title = Words.couldNotConnect + dialog.getHostname();
                    JOptionPane.showMessageDialog(dialog, Words.couldNotConnectFurther, title, JOptionPane.WARNING_MESSAGE);
                } catch (IOException ioe) {
                    // this should be impossible
                    ioe.printStackTrace();
                } catch (InterruptedException ie) {
                    // this should also be impossible
                    ie.printStackTrace();
                }
            }
        } // if

        // start a new game if connection is made successfully
        if (server.isConnected()) {
            if (dialog.getNewGameFlag()) {
                server.newGame(dialog.getGameName(), dialog.getRandomizeFlag(), dialog.getPrivacyFlag());
            } else {
                server.joinGame(dialog.getGameName());
            } // if( new game )

        } // if( connected )

    } // actionPerformed( ActionEvent )

} // RiskNetworkDialogListener.java
