// Protocol.java

package uk.org.redfelineninja.risk.net;

/**
 * This class contains all the commands used in the Risk newtwork proocol.
 * By gathering it all together in one class there is no possibility of typing
 * errors within main source code. Since the server error messages are specified
 * as protocol it allows the client to trap specific error messages and display
 * them in a frame as appropriate.
 */
public class Protocol {
    // HIGH LEVEL CLASSIFICATION

    public static final char command = '#';
    public static final char comment = '%';
    public static final char query = '?';
    public static final char warning = '!';
    public static final char chatCommand = '/';

    // CLIENT COMMANDS

    public static final String cGameOver = "finish"; // <player> [<string>]
    public static final String cLostPlayer = "lost"; // <player>
    public static final String cNewGame = "new"; // <player> ...
    public static final String cNewPlayer = "player"; // <player> <int>
    public static final String cNewSpectator = "spectator"; // <player>
    public static final String cSetArmies = "armies"; // <str> <int>
    public static final String cSetCards = "cards"; // <country> ...
    public static final String cSetDice = "dice"; // <int> ...
    public static final String cSetOwner = "country"; // <str> <str>
    public static final String cSetTurn = "turn"; // <player>
    public static final String cSelect = "select"; // <country>
    public static final String cShowStatus = "status"; // <string>

    // SERVER COMMANDS
    public static final String sPrompt = "--";
    public static final String sIdentify = "player"; // <str> <bool> <bool>
    public static final String sNewGame = "new"; // <str> <int>
    public static final String sJoinGame = "join"; // <str>
    public static final String sWatchGame = "watch"; // <str>
    public static final String sStartGame = "start";

    public static final String sCountryClicked = "click"; // <str>
    public static final String sFortify = "fortify";
    public static final String sFinishTurn = "finish";
    
    public static final String sCheat = "cheat";

    public static final String sCards = "cards"; // <str> ...
    public static final String sLeaveGame = "leave";
    public static final String sStopGame = "stop"; // <str>

    // SERVER WARNINGS
    public static final String wGameStarted = "Game Started";
    public static final String wServerError = "Internal Server Error";
    public static final String wNoGame = "No game has been started";
    public static final String wOutOfTurn = "It is not your turn";
    public static final String wNoCountry = "No country is selected";
    public static final String wNotAdjacent = "You can not attack that country from there";
    public static final String wNotOwner = "You do not own that country";
    public static final String wNotAllowed = "You are not allowed to do that";
    public static final String wInvalidCommand = "Invalid or unsupported command";
    public static final String wAlreadyRunning = "There is already a game running";
    public static final String wTerminated = "Game terminated. There are no more users.";

    // SLASH COMMANDS
    public static final String sWho = "who";
    public static final String sRobot = "robot";
    public static final String sOn = "on";
    public static final String sOff = "off";

    // SLASH REPLIES
    public static final String rBadCommand = "Bad command";
    public static final String rBadArguments = "Bad arguments";
    public static final String rLoggedOn = " Currently logged on are:-";
    public static final String rRobotHints = " Robot hints are now "; // on/off

    // GENERAL OUTPUT
    public static final String gVanquished = /* player */
    " has been forever vanquished";
    public static final String gAlreadyRunning = "There is already a game running";
    public static final String gBugReport =
        "\nPlease mail the following to <D.Thompson@gmx.net> and include Risk in the\nsubject line.\n";

    // ARGUMENTS
    public static final String aRandom = "random";
    public static final String aPrivacy = "privacy";
} // Protocol.java
