// Dice.java

package uk.org.redfelineninja.risk.server;

import uk.org.redfelineninja.risk.net.RiskClientInterface;

/**
 * This class implements tha dice battling mechanism of risk.
 */
public class Dice {
    // WARNING changing these WILL break code lower down (attackHelper at least)
    private static final int maxAtt = 3;
    private static final int maxDef = 2;

    // return values
    public static final int OK = 0;
    public static final int CANT_ATTACK = -1;
    public static final int ILLEGAL_ATTACK = -2;

    private RiskClientInterface client;

    public Dice(RiskClientInterface c) {
        client = c;
    }

    /*
     * Commented out after testing
     */
    public static void main(String args[]) {
        Dice d = new Dice(null);

        int[] die = d.rollDice(40);

        StringBuffer sb = new StringBuffer();

        for (int i = 0; i < die.length; i++) {
            sb.append(die[i]).append(' ');
        }

        System.out.println(sb.toString());
    }
    /**/

    public int[] rollDice(int n) {
        int[] die = new int[n];

        for (int i = 0; i < n; i++) {
            die[i] = (int) Math.ceil(Math.random() * 6);
        }

        // tell the client about the roll
        client.setDice(die);

        return die;
    }

    public int bestAttack(RiskEngine.Country att, RiskEngine.Country def) {
        if ((att.getArmies() < 1) || (def.getArmies() < 0)) {
            // we can not make an attack
            return CANT_ATTACK;
        }

        // it is always best to use the most dice possible

        // attacker may use all but one of his armies
        int attDice = Math.min(att.getArmies() - 1, maxAtt);
        if (attDice < 1)
            attDice = 1;

        // defender can use all armies
        int defDice = Math.min(def.getArmies(), maxDef);
        if (defDice < 1)
            defDice = 1;

        attackHelper(att, attDice, def, defDice);
        return OK;
    }

    public int attack(RiskEngine.Country att, int attDice, RiskEngine.Country def, int defDice) {
        if ((att.getArmies() < 1) || (def.getArmies() < 0)) {
            // we can not make an attack
            return CANT_ATTACK;
        }

        int attMaxDice = Math.min(att.getArmies() - 1, maxAtt);
        int defMaxDice = Math.min(def.getArmies(), maxDef);

        if ((attDice > attMaxDice) || (defDice > defMaxDice)) {
            // illegal arguments
            return ILLEGAL_ATTACK;
        }

        // passed sanity tests
        attackHelper(att, attDice, def, defDice);
        return OK;
    }

    /**
     * Performs an attack, sanity checking is performed by public methods.
     */
    private void attackHelper(RiskEngine.Country att, int attDice, RiskEngine.Country def, int defDice) {
        int[] die = rollDice(attDice + defDice);

        int maxA, midA, maxD, minD, attLoss = 0, defLoss = 0;

        // swap top and bottom die so that the larger is at the top
        if (die[0] < die[attDice - 1]) {
            midA = die[0];
            die[0] = die[attDice - 1];
            die[attDice - 1] = midA;
        }

        // set maxA and midA appropriatly
        if (attDice == 3) {
            if (die[0] < die[1]) {
                maxA = die[1];
                midA = Math.max(die[0], die[2]);
            } else {
                maxA = die[0];
                midA = Math.max(die[0], die[2]);
            }
        } else {
            maxA = Math.max(die[0], die[attDice - 1]);
            midA = Math.min(die[0], die[attDice - 1]);
        }

        // set maxD (if there is only one dice then attDice == die.length
        maxD = Math.max(die[attDice], die[die.length - 1]);

        // set minD
        minD = Math.min(die[attDice], die[die.length - 1]);

        // battles

        // first battle
        if (maxA > maxD) {
            // attacker wins
            defLoss++;
        } else {
            // defender wins
            attLoss++;
        }

        // is there a second battle
        if ((attDice > 1) && (defDice > 1)) {
            if (midA > minD) {
                // attacker wins
                defLoss++;
            } else {
                // defender wins
                attLoss++;
            }

        }

        // make the changes
        if (defLoss != 0) {
            def.setArmies(def.getArmies() - defLoss);
        }

        if (attLoss != 0) {
            att.setArmies(att.getArmies() - attLoss);
        }

    } // attackHelper( ... )
} // Dice.class
