// BroadcastWriter.java

package uk.org.redfelineninja.risk.test;

import java.io.*;
import java.util.*;

/**
 * This class allows the caller to agregate many Writer into one single one.
 *
 * In addition it can be interegated by name to return a Writer of that name.
 */
public class BroadcastWriter extends Writer {
    private Vector writers = new Vector();

    public void addWriter(Writer w) {
        writers.addElement(w);
    }

    public boolean removeWriter(Writer w) {
        return writers.removeElement(w);
    }

    public void write(char cbuf[], int off, int len) throws IOException {
        boolean failed = false;

        for (Enumeration e = writers.elements(); e.hasMoreElements();) {
            try {
                ((Writer) e.nextElement()).write(cbuf, off, len);
            } catch (IOException ioe) {
                failed = true;
            }
        }

        if (failed)
            throw new IOException("Some of my writers could not be written to");
    } // write( char[], int, int )

    public void flush() throws IOException {
        boolean failed = false;

        for (Enumeration e = writers.elements(); e.hasMoreElements();) {
            try {
                ((Writer) e.nextElement()).flush();
            } catch (IOException ioe) {
                failed = true;
            }
        }

        if (failed)
            throw new IOException("Some of my writers could not be flushed");
    } // flush()

    public void close() throws IOException {
        for (Enumeration e = writers.elements(); e.hasMoreElements();) {
            ((Writer) e.nextElement()).close();
        }

        // dispose of writers, our caller may not forget us
        writers = null;
    } // close()

} // end class   
